#include "BackedPanel.h"
#include <wx/dcbuffer.h>

BEGIN_EVENT_TABLE(BackedPanel, wxPanel)
	EVT_PAINT(BackedPanel::OnPaint)
END_EVENT_TABLE()
BEGIN_EVENT_TABLE(BackedPanelWithGrid, BackedPanel)
	EVT_PAINT(BackedPanelWithGrid::OnPaint)
	EVT_MOUSE_EVENTS(BackedPanelWithGrid::OnMouse)
END_EVENT_TABLE()

void BackedPanel::OnPaint(wxPaintEvent &event)
{
	if (mybitmap.IsOk())
	{
		SetBackgroundStyle(wxBG_STYLE_CUSTOM);
		wxBufferedPaintDC mydc(this,mybitmap);
	}
	else
	{
		SetBackgroundStyle(wxBG_STYLE_SYSTEM);
		wxPaintDC mydc(this);
	}
	
}

void BackedPanelWithGrid::drawcursor(wxDC &dc, const Cursor &c, const Cursor &g)
{
	if (c.visible)
	{
		int x0=c.x*g.w-1;
		int y0=c.y*g.h-1;
		int w=c.w*g.w+2;
		int h=c.h*g.h+2;
		wxBrush highlightbrush(wxColour(c.r,c.g,c.b,c.a));
		dc.SetPen(*wxTRANSPARENT_PEN);
		dc.SetBrush(highlightbrush);
		//top
		dc.DrawRectangle(x0,y0,w,c.thickness);
		//left
		dc.DrawRectangle(x0,y0,c.thickness,h);
		//bottom
		dc.DrawRectangle(x0,y0+h-c.thickness,w,c.thickness);
		//right
		dc.DrawRectangle(x0+w-c.thickness,y0,c.thickness,h);
		/*
		int thickfactor=c.thickness/2;
		int x0=c.x*g.w-1+thickfactor;
		int y0=c.y*g.h-1+thickfactor;
		int w=c.w*g.w+2-thickfactor;
		int h=c.h*g.h+2-thickfactor;
		wxPen highlightpen(wxColour(c.r,c.g,c.b,c.a),c.thickness);
		dc.SetPen(highlightpen);
		dc.SetBrush(*wxTRANSPARENT_BRUSH);
		dc.DrawRectangle(x0,y0,w,h);
		*/
	}
}


void BackedPanelWithGrid::draw(wxDC &dc, const Grid &mygrid)
{
	const bool draw_crap=false;
	if (draw_crap)
	{
		int r=rand()%256;
		int g=rand()%256;
		int b=rand()%256;
		dc.SetBrush(wxBrush(wxColour(r,g,b)));
		dc.SetPen(*wxTRANSPARENT_PEN);
		dc.DrawRectangle(0,0,256,256);
		return;
	}
	
	//Draw grid
	const Cursor &g=mygrid.grid;
	if (g.visible && g.h>0 && g.w>0)
	{
		wxPen gridpen(wxColour(g.r,g.g,g.b,g.a),g.thickness);
		dc.SetPen(gridpen);
		//draw horizontal grid lines
		{
			int x0 = 0, x1=mygrid.width;
			for (int y=0;y<mygrid.height;y+=g.h)
			{
				dc.DrawLine(x0,y,x1,y);
			}
		}
		//draw vertical grid lines
		{
			int y0 = 0, y1=mygrid.width;
			for (int x=0;x<mygrid.width;x+=g.w)
			{
				dc.DrawLine(x,y0,x,y1);
			}
		}
	}
	drawcursor(dc,mygrid.highlight,g);
	int i;
	const vector<Cursor> &cursors=mygrid.cursors;
	for (i=0;i<(int)cursors.size();i++)
	{
		const Cursor &c=cursors[i];
		drawcursor(dc,c,g);
	}
}


void BackedPanelWithGrid::OnPaint(wxPaintEvent &event)
{
	if (mybitmap.IsOk())
	{
		SetBackgroundStyle(wxBG_STYLE_CUSTOM);
		wxPaintDC mydc(this);
		mydc.DrawBitmap(mybitmap,0,0);
		draw(mydc,mygrid);
	}
	else
	{
		SetBackgroundStyle(wxBG_STYLE_SYSTEM);
		wxPaintDC mydc(this);
		draw(mydc,mygrid);
	}	
}

void BackedPanelWithGrid::MoveCursor(Cursor &oldcursor, const Cursor &newcursor)
{
//	if (oldcursor==newcursor) return;
	int gw=mygrid.grid.w;
	int gh=mygrid.grid.h;
	if (oldcursor.visible)
	{
		int x0,y0,w,h;
		x0=oldcursor.x*gw-1;
		w=oldcursor.w*gw+2;
		y0=oldcursor.y*gh-1;
		h=oldcursor.h*gh+2;
		this->RefreshRect(wxRect(x0,y0,w,h));
	}
	if (newcursor.visible)
	{
		int x0,y0,w,h;
		x0=newcursor.x*gw-1;
		w=newcursor.w*gw+2;
		y0=newcursor.y*gh-1;
		h=newcursor.h*gh+2;
		this->RefreshRect(wxRect(x0,y0,w,h));
	}
	oldcursor=newcursor;
}

void BackedPanelWithGrid::UpdateHighlight(int x,int y,bool offscreen)
{
	Cursor &h=mygrid.highlight;
	if (x<0 || x>=mygrid.width || y<0 || y>=mygrid.height) offscreen=true;
	if (offscreen)
	{
		if (h.visible)
		{
			Cursor c=h;
			c.visible=false;
			MoveCursor(h,c);
		}
	}
	if (!offscreen)
	{
		int gw=mygrid.grid.w;
		int gh=mygrid.grid.h;

		int gx=x/gw;
		int gy=y/gh;

		Cursor c=mygrid.highlight;
		c.x=gx;
		c.y=gy;
		c.visible=true;
		if (c!=h)
		{
			MoveCursor(h,c);
		}
	}
}

void BackedPanelWithGrid::OnMouse(wxMouseEvent &event)
{
	Cursor &g=mygrid.grid;
	Cursor &h=mygrid.highlight;
	int x=event.m_x;
	int y=event.m_y;
	bool offscreen=event.Leaving();
	if (!offscreen)
	{
		last_mousex=x;
		last_mousey=y;
		last_gridx=x/g.w;
		last_gridy=y/g.h;
	}
	else
	{
		last_mousex=-1;
		last_mousey=-1;
		last_gridx=-1;
		last_gridy=-1;
	}

	if (!highlightvisible)
	{
		if (h.visible)
		{
			Cursor c=h;
			c.visible=false;
			MoveCursor(h,c);
		}
		return event.Skip();
	}
	UpdateHighlight(x,y,offscreen);
	event.Skip();
}

void BackedPanelWithGrid::setgrid(int gw, int gh, int nw, int nh)
{
	if (nw==-1 || nh==-1)
	{
		GetSize(&nw,&nh);
	}
	mygrid.width=nw;
	mygrid.height=nh;
	Cursor &g=mygrid.grid;
	g.w=gw;
	g.h=gh;
	Cursor &h=mygrid.highlight;
	UpdateHighlight(last_mousex,last_mousey);
	Refresh();
}
