#include "GUI_common.h"
#include "BP.h"

void draw_vram(const vector<bitmap> &tiles,wxBitmap &bmp, bool includebottomrows)
{
	bitmap b;
	int VRAM_LAST_ROW;
	if (includebottomrows)
	{
		VRAM_LAST_ROW=32;
		b.resize(256,256);
	}
	else
	{
		VRAM_LAST_ROW=24;
		b.resize(256,192);
	}
	int W,H;
	W=b.getWidth();
	H=b.getHeight();
	int row,col;
	int half;
	for (half=0;half<2;half++)	
	{
		for (row=0;row<VRAM_LAST_ROW;row++)
		{
			int y0=row*8;
			for (col=0;col<16;col++)
			{
				int x0=col*8 + half*128;
				int i=half*0x200+col+row*16;
				b.blit(tiles[i],x0,y0,255);
			}
		}
	}
	u8 mypalette[]=
	{
		255,255,255,170,170,170,85,85,85,0,0,0
	};

	u8 *data = b.make24bit(mypalette);
	wxImage img(W,H,data,false);
	wxBitmap newbmp(img);
	bmp=newbmp;
}

void draw_palette(const vector<u8> &rgbpalette,wxBitmap &bmp)
{
	if (!bmp.IsOk())
	{
		bmp.Create(128,64);
	}

	wxMemoryDC dc(bmp);
	dc.SetPen(*wxTRANSPARENT_PEN);
	int y,x;
	for (y=0;y<4;y++)
	{
		for (x=0;x<8;x++)
		{
			int i=x*4+y;
			int r,g,b;
			r=rgbpalette[i*3+0];
			g=rgbpalette[i*3+1];
			b=rgbpalette[i*3+2];
			wxBrush mybrush(wxColor(r,g,b));
			dc.SetBrush(mybrush);
			dc.DrawRectangle(x*16,y*16,16,16);
		}
	}
}

void draw_screen(const vector<u8> &rgbpalette, const vector<bitmap> &tiles, CBP map_vram, wxBitmap &bmp)
{
	bitmap map(256,256);
	CBP map_tiles=map_vram;
	CBP map_attrib=map_vram+0x2000;
	
	int i;
	for (i=0;i<0x400;i++)
	{
		int x,y;
		x=(i&0x1F)*8;
		y=(i/0x20)*8;
		int t,a;
		t=map_tiles[i];
		a=map_attrib[i];
		if (t<0x80) t+=0x100;
		if (a&0x08) t+=0x200;
		int c;
		c=a&7;
		bitmap mytile=tiles[t];
		mytile.add(c*4);
		if (a&0x20) mytile.hflip();
		if (a&0x40) mytile.vflip();
		map.blit(mytile,x,y,255);
	}
	u8 *data = map.make24bit(&rgbpalette[0]);
	wxImage img(map.getWidth(),map.getHeight(),data,false);
	wxBitmap newbmp(img);
	bmp=newbmp;
}

void draw_tileset(const vector<u8> &palette, const vector<bitmap> &maptiles, wxBitmap &bmp)
{
	bitmap b;
	MyApp::drawtileset(b,maptiles);

	u8 *data=b.make24bit(&palette[0]);
	wxImage img(256,256,data);
	wxBitmap newbmp(img);
	bmp=newbmp;
}
