#ifndef __BITMAP_H__
#define __BITMAP_H__

#pragma once

#include "includes.h"
#include <png.h>

#include <vector>

#define BITMAP_BOUNDS_CHECKING 0

using std::vector;

template<class PIXEL_T>
struct _bitmap_row
{
	static PIXEL_T garbage;
	vector<PIXEL_T> *data;
	PIXEL_T& operator[](unsigned int i)
	{
#if BITMAP_BOUNDS_CHECKING
		if (data)
		{
			if (i>=0 && i<(*data).size())
			{
				return (*data)[i];
			}
		}
		return garbage;
#else
		return (*data)[i];
#endif
	}
	void attach(vector<PIXEL_T> &attachthis)
	{
		data=&attachthis;
	}
	_bitmap_row(vector<PIXEL_T> &attachthis)
	{
		attach(attachthis);
	}
	_bitmap_row()
	{
		data=NULL;
	}
};

template<class PIXEL_T>
struct _const_bitmap_row
{
	const vector<PIXEL_T> *data;
	const PIXEL_T& operator[](unsigned int i) const
	{
#if BITMAP_BOUNDS_CHECKING
		if (data)
		{
			if (i>=0 && i<(*data).size())
			{
				return (*data)[i];
			}
		}
		return 0;
#else
		return (*data)[i];
#endif
	}
	void attach(const vector<PIXEL_T> &attachthis)
	{
		data=&attachthis;
	}
	_const_bitmap_row(const vector<PIXEL_T> &attachthis)
	{
		attach(attachthis);
	}
	_const_bitmap_row()
	{
		data=NULL;
	}
};


template <class PIXEL_T>
struct _bitmap
{
	vector<vector<PIXEL_T> >data;
	unsigned int getWidth() const
	{
		if (data.size()==0) return 0;
		return data[0].size();
	}
	unsigned int getHeight() const
	{
		return data.size();
	}
	_bitmap()
	{
		
	}
	_bitmap(int newWidth, int newHeight)
	{
		resize(newWidth,newHeight);
	}
	void resize(unsigned int newWidth, unsigned int newHeight)
	{
		data.clear();
		unsigned int i;
		data.resize(newHeight);
		for (i=0;i<newHeight;i++)
		{
			data[i].resize(newWidth);
		}
	}
	_bitmap_row<PIXEL_T> operator[](unsigned int y)
	{
#if BITMAP_BOUNDS_CHECKING
		if (y>=0 && y<data.size())
		{
			return _bitmap_row<PIXEL_T>(data[y]);
		}
		return _bitmap_row<PIXEL_T>();
#else
		return _bitmap_row<PIXEL_T>(data[y]);
#endif
	}
	_const_bitmap_row<PIXEL_T> operator[](unsigned int y) const
	{
#if BITMAP_BOUNDS_CHECKING
		if (y>=0 && y<data.size())
		{
			return _const_bitmap_row<PIXEL_T>(data[y]);
		}
		return _const_bitmap_row<PIXEL_T>();
#else
		return _const_bitmap_row<PIXEL_T>(data[y]);
#endif
	}
	void clear(PIXEL_T clearcolor=0)
	{
		_bitmap<PIXEL_T> &b=*this;
		int x,y;
		int W,H;
		W=b.getWidth();
		H=b.getHeight();
		for (x=0;x<W;x++)
		{
			for (y=0;y<H;y++)
			{
				b[y][x]=clearcolor;
			}
		}
	}
	void blit(const _bitmap<PIXEL_T> &src, int x, int y, PIXEL_T TRANSPAR=0)
	{
		blit(*this,src,x,y,TRANSPAR);
	}
	void blit(const _bitmap<PIXEL_T> &src, int dest_x, int dest_y, int src_x, int src_y, int width, int height, PIXEL_T TRANSPAR=0)
	{
		blit(*this,src,dest_x,dest_y,src_x,src_y,width,height,TRANSPAR);
	}
	static void blit(_bitmap<PIXEL_T> &dest, const _bitmap<PIXEL_T>&src, int dest_x_base, int dest_y_base, int src_x_base, int src_y_base, int width, int height, PIXEL_T TRANSPAR=0);
	static void blit(_bitmap<PIXEL_T> &dest, const _bitmap<PIXEL_T>&src, int dest_x, int dest_y, PIXEL_T TRANSPAR=0);
	bool boxInBounds(int x0, int y0, int x1, int y1) const
	{
		int W=getWidth();
		int H=getHeight();
		if (x0<0 || x0>=W || x1<0 || x1>=W || y0<0 || y0>=H || y1<0 || y1>=H)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	void drawBox(int x0, int y0, int x1, int y1, PIXEL_T boxcolor)
	{
		drawBox(*this,x0,y0,x1,y1,boxcolor);
	}
	static void drawBox(_bitmap<PIXEL_T> &b, int x0, int y0, int x1, int y1, PIXEL_T boxcolor);
	void drawBorder(PIXEL_T boxcolor)
	{
		drawBox(0,0,getWidth()-1,getHeight()-1,boxcolor);
	}
	u8* make24bit(const u8 *palette) const;
	u8* make24bit() const;
	void make32bit(_bitmap<u32> &out, const u8 *palette) const;
	void hflip();
	void vflip();
	void add(PIXEL_T addthis);
	bool write_png(const _TCHAR *filename, const vector<u8> &palette) const;
	bool write_png(const _TCHAR *filename) const;

};

inline void clip_left_right(int my_left, int my_width, int container_left, int container_width, int &clipleft, int &clipright);
inline int int_max(int a, int b);

template<class PIXEL_T>
void _bitmap<PIXEL_T>::drawBox(_bitmap<PIXEL_T> &b, int x0, int y0, int x1, int y1, PIXEL_T boxcolor)
{
	int H=b.getHeight();
	int W=b.getWidth();
	
	if (!b.boxInBounds(x0,y0,x1,y1)) return;

	for (int y=y0;y<=y1;y++)
	{
		b[y][x0]=boxcolor;
		b[y][x1]=boxcolor;
	}
	for (int x=x0;x<=x1;x++)
	{
		b[y0][x]=boxcolor;
		b[y1][x]=boxcolor;
	}
}

inline void clip_left_right(int my_left, int my_width, int container_left, int container_width, int &clipleft, int &clipright)
{
	clipleft=0;
	clipright=0;
	
	int my_right=my_left+my_width;
	int container_right=container_left+container_width;

	if (my_right>container_right)
	{
		clipright=my_right-container_right;
	}
	if (my_left<container_left)
	{
		clipleft=container_left-my_left;
	}
}

inline int int_max(int a, int b)
{
	if (a>b) return a; else return b;
}

template<class PIXEL_T>
void _bitmap<PIXEL_T>::blit(_bitmap<PIXEL_T> &dest, const _bitmap<PIXEL_T> &src, int dest_x_base, int dest_y_base, int src_x_base, int src_y_base, int width, int height,PIXEL_T TRANSPAR)
{
	int dest_width=dest.getWidth();
	int dest_height=dest.getHeight();
	int src_width=src.getWidth();
	int src_height=src.getHeight();
	
	int src_clip_left=0;
	int src_clip_right=0;
	int src_clip_top=0;
	int src_clip_bottom=0;
	int dest_clip_left=0;
	int dest_clip_right=0;
	int dest_clip_top=0;
	int dest_clip_bottom=0;

	clip_left_right(src_x_base,width,0,src_width,src_clip_left,src_clip_right);
	clip_left_right(src_y_base,height,0,src_height,src_clip_top,src_clip_bottom);
	clip_left_right(dest_x_base,width,0,dest_width,dest_clip_left,dest_clip_right);
	clip_left_right(dest_y_base,height,0,dest_height,dest_clip_top,dest_clip_bottom);

	int area_clip_left=int_max(src_clip_left,dest_clip_left);
	int area_clip_right=int_max(src_clip_right,dest_clip_right);
	int area_clip_top=int_max(src_clip_top,dest_clip_top);
	int area_clip_bottom=int_max(src_clip_bottom,dest_clip_bottom);
	int area_width=width-area_clip_left-area_clip_right;
	int area_height=height-area_clip_top-area_clip_bottom;
	if (area_width<=0 || area_height<=0) return;

	int src_x_offset=area_clip_left+src_x_base;
	int dest_x_offset=area_clip_left+dest_x_base;
	int src_y_offset=area_clip_top+src_y_base;
	int dest_y_offset=area_clip_top+dest_y_base;

	for (int y=0;y<area_height;y++)
	{
		for (int x=0;x<area_width;x++)
		{
			int sy=y+src_y_offset;
			int sx=x+src_x_offset;
			int dy=y+dest_y_offset;
			int dx=x+dest_x_offset;

			assert (sx>=0 && sx<src_width && sy>=0 && sy<src_height && dx>=0 && dx<dest_width && dy>=0 && dy<dest_height);
			PIXEL_T c=src[sy][sx];
			if (c!=TRANSPAR)
			{
				dest[dy][dx]=c;
			}
		}
	}
}

template<class PIXEL_T>
void _bitmap<PIXEL_T>::blit(_bitmap<PIXEL_T> &dest, const _bitmap<PIXEL_T> &src, int dest_x, int dest_y,PIXEL_T TRANSPAR)
{
	blit(dest,src,dest_x,dest_y,0,0,src.getWidth(),src.getHeight(),TRANSPAR);
}

template<class PIXEL_T>
u8* _bitmap<PIXEL_T>::make24bit(const u8 *palette) const
{
	int W,H;
	W=getWidth();
	H=getHeight();
	if (W<=0 || H<=0 ) return NULL;
	int memsize=W*H*3;
	u8 *data_base=(u8*)malloc(memsize);
	u8 *data=data_base;
	for (int y=0;y<H;y++)
	{
		for (int x=0;x<W;x++)
		{
			int c=(*this)[y][x];
			const u8 *pe=palette+c*3;
			*data++=*pe++;
			*data++=*pe++;
			*data++=*pe++;
		}
	}
	return data_base;
}
template<class PIXEL_T>
u8* _bitmap<PIXEL_T>::make24bit() const
{
	int W,H;
	W=getWidth();
	H=getHeight();
	if (W<=0 || H<=0 ) return NULL;
	int memsize=W*H*3;
	u8 *data_base=(u8*)malloc(memsize);
	u8 *data=data_base;
	for (int y=0;y<H;y++)
	{
		for (int x=0;x<W;x++)
		{
			int c=(*this)[y][x];
			*data++=((c>>0)&0xFF);
			*data++=((c>>8)&0xFF);
			*data++=((c>>16)&0xFF);
		}
	}
	return data_base;
}
template<class PIXEL_T>
void _bitmap<PIXEL_T>::make32bit(_bitmap<u32> &out,const u8 *palette) const
{
	int W,H;
	W=getWidth();
	H=getHeight();
	if (W<=0 || H<=0 )return;
	out.resize(W,H);
	for (int y=0;y<H;y++)
	{
		for (int x=0;x<W;x++)
		{
			int c=(*this)[y][x];
			const u8 *pe=palette+c*3;
			int r=*pe++;
			int g=*pe++;
			int b=*pe++;
			out[y][x]=(r)+(g<<8)+(b<<16);
		}
	}
}

template<class PIXEL_T>
void _bitmap<PIXEL_T>::hflip()
{
	PIXEL_T temp;
	int W=getWidth();
	int W2=W/2;
	int H=getHeight();
	for (int y=0;y<H;y++)
	{
		for (int x=0;x<W2;x++)
		{
			temp=(*this)[y][x];
			(*this)[y][x]=(*this)[y][W-x-1];
			(*this)[y][W-x-1]=temp;
		}
	}
}
template<class PIXEL_T>
void _bitmap<PIXEL_T>::vflip()
{
	PIXEL_T temp;
	int W=getWidth();
	int H=getHeight();
	int H2=W/2;
	for (int y=0;y<H2;y++)
	{
		for (int x=0;x<W;x++)
		{
			temp=(*this)[y][x];
			(*this)[y][x]=(*this)[H-y-1][x];
			(*this)[H-y-1][x]=temp;
		}
	}
}
template<class PIXEL_T>
void _bitmap<PIXEL_T>::add(PIXEL_T addthis)
{
	int W=getWidth();
	int H=getHeight();
	for (int y=0;y<H;y++)
	{
		for (int x=0;x<W;x++)
		{
			(*this)[y][x]+=addthis;
		}
	}
}

typedef _bitmap<u8> bitmap;
typedef _bitmap<u32> bitmap32;

bool write_png_file(const bitmap &bmp, const vector<u8> &palette, const _TCHAR *filename);
bool write_png_file(const bitmap32 &bmp, const _TCHAR *filename);

template<class PIXEL_T>
bool _bitmap<PIXEL_T>::write_png(const _TCHAR *filename, const vector<u8> &palette) const
{
	const bitmap &bmp=*this;
	return ::write_png_file(bmp,palette,filename);
}

template<class PIXEL_T>
bool _bitmap<PIXEL_T>::write_png(const _TCHAR *filename) const
{
	const bitmap32 &bmp=*this;
	return ::write_png_file(bmp,filename);
}


#endif

